-------------------------------------------------------------------------------- 
-- Copyright (c) 2004 Xilinx, Inc. 
-- All Rights Reserved 
-------------------------------------------------------------------------------- 
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /   Vendor: Xilinx 
-- \   \   \/    Author: Latha Pillai, Advanced Product Group, Xilinx, Inc.
--  \   \        Filename: FIR_MCF6
--  /   /        Date Last Modified:  June 23, 2004 
-- /___/   /\    Date Created: June 23, 2004 
-- \   \  /  \ 
--  \___\/\___\ 
-- 
--
-- Revision History: 
-------------------------------------------------------------------------------- 
--   
--   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" 
--   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND 
--   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, 
--   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, 
--   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION 
--   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT, 
--   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE 
--   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY 
--   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE 
--   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR 
--   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF 
--   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--   FOR A PARTICULAR PURPOSE. 
--
-------------------------------------------------------------------------------- 

--check opmode settings and m and p reg , register settings

--LIBRARY -work;
library IEEE; 
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use IEEE.numeric_std.all;

ENTITY FIR_MCF6 IS
   PORT (
      SLICE_CLK               : IN std_logic;   
      RST                     : IN std_logic;   
      A_IN, B_IN, C_IN        : IN std_logic_vector(7 DOWNTO 0);   
      D_IN, E_IN, F_IN        : IN std_logic_vector(7 DOWNTO 0);   
     -- C1_IN, C2_IN, C3_IN, C4_IN      : IN std_logic_vector(7 DOWNTO 0);
     -- C5_IN, C6_IN, C7_IN, C8_IN      : IN std_logic_vector(7 DOWNTO 0);   
      OUT1, OUT2, OUT3                : OUT std_logic_vector(9 DOWNTO 0);   
      OUT4, OUT5, OUT6                : OUT std_logic_vector(9 DOWNTO 0));   
END FIR_MCF6;

ARCHITECTURE FIR_MCF6 OF FIR_MCF6 IS

   COMPONENT FIR_SLICE1
      PORT (
         CLK                     : IN  std_logic;
         RST                     : IN std_logic;
         A_IN                    : IN  std_logic_vector(7 DOWNTO 0);
         C_IN                    : IN  std_logic_vector(7 DOWNTO 0);
         PCOUT                   : OUT std_logic_vector(47 DOWNTO 0);
         POUT                    : OUT std_logic_vector(47 DOWNTO 0));
   END COMPONENT;


   COMPONENT FIR_SLICE2
      PORT (
         CLK                     : IN  std_logic;
         RST                     : IN std_logic;
         A_IN                    : IN  std_logic_vector(7 DOWNTO 0);
         C_IN                    : IN  std_logic_vector(7 DOWNTO 0);
         PCIN                    : IN  std_logic_vector(47 DOWNTO 0);
         PCOUT                   : OUT std_logic_vector(47 DOWNTO 0);
         POUT                    : OUT std_logic_vector(47 DOWNTO 0));
   END COMPONENT;
	
	
   COMPONENT SRL18_7
      PORT (
         CLK                     : IN  std_logic;
         RST                     : IN std_logic;
         D                       : IN  std_logic_vector(7 DOWNTO 0);
         Q                       : OUT std_logic_vector(7 DOWNTO 0));
   END COMPONENT;


   SIGNAL A_IN_REG, B_IN_REG, C_IN_REG               :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL D_IN_REG, E_IN_REG, F_IN_REG               :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL MUX_OUT1A, MUX_OUT1B                       :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL MUX_OUT1C, MUX_OUT1D                       :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL MUX_OUT2A, MUX_OUT2B                       :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL MUX_OUT                                    :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL SUM_INT1, SUM_INT2, SUM_INT3, SUM_INT4     :  std_logic_vector(47 DOWNTO 0);   
   SIGNAL SUM_INT5, SUM_INT6, SUM_INT7, SUM_INT8     :  std_logic_vector(47 DOWNTO 0);   
   SIGNAL Q_OUT1, Q_OUT2, Q_OUT3, Q_OUT4             :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL Q_OUT5, Q_OUT6, Q_OUT7, Q_OUT8             :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL OUTPUT_EN                                  :  std_logic;   
   SIGNAL DLY_CNTR                                   :  std_logic_vector(5 DOWNTO 0);   
   SIGNAL C1_IN, C2_IN, C3_IN, C4_IN                 :  std_logic_vector(7 DOWNTO 0); 
   SIGNAL C5_IN, C6_IN, C7_IN, C8_IN                 :  std_logic_vector(7 DOWNTO 0);
   SIGNAL IN_MUX_SEL1, IN_MUX_SEL2                   :  std_logic;   
   SIGNAL IN_MUX_SEL3, IN_MUX_SEL4                   :  std_logic;   
   SIGNAL IN_MUX_SEL5, IN_MUX_SEL6                   :  std_logic;   
   SIGNAL OUT_MUX_SEL1, OUT_MUX_SEL2                 :  std_logic;   
   SIGNAL OUT_MUX_SEL3, OUT_MUX_SEL4                 :  std_logic;   
   SIGNAL OUT_MUX_SEL5, OUT_MUX_SEL6                 :  std_logic;   
   SIGNAL OUT_SEL                                    :  std_logic_vector(7 DOWNTO 0);   
   SIGNAL SUM_INT8_REG                               :  std_logic_vector(9 DOWNTO 0);   
   SIGNAL MUX_SEL12, MUX_SEL34, MUX_SEL56            :  std_logic_vector(1 DOWNTO 0);   

   --
   -- Slice Chain. Each slice corresponds to 1 Tap
   --          

BEGIN

MUX_SEL12 <= IN_MUX_SEL1 & IN_MUX_SEL2;
MUX_SEL34 <= IN_MUX_SEL3 & IN_MUX_SEL4;
MUX_SEL56 <= IN_MUX_SEL5 & IN_MUX_SEL6;

   
   PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         IN_MUX_SEL1 <= '0';    
         IN_MUX_SEL2 <= '0';    
         IN_MUX_SEL3 <= '0';    
         IN_MUX_SEL4 <= '0';    
         IN_MUX_SEL5 <= '0';    
         IN_MUX_SEL6 <= '1';    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         IN_MUX_SEL1 <= IN_MUX_SEL6;    
         IN_MUX_SEL2 <= IN_MUX_SEL1;    
         IN_MUX_SEL3 <= IN_MUX_SEL2;    
         IN_MUX_SEL4 <= IN_MUX_SEL3;    
         IN_MUX_SEL5 <= IN_MUX_SEL4;    
         IN_MUX_SEL6 <= IN_MUX_SEL5;    
      END IF;
   END PROCESS;

   PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         C1_IN <= "00000000"; C2_IN <= "00000000"; 
         C3_IN <= "00000000"; C4_IN <= "00000000"; 
         C5_IN <= "00000000"; C6_IN <= "00000000"; 
         C7_IN <= "00000000"; C8_IN <= "00000000"; 
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         C1_IN <= "10101010"; 
         C2_IN <= "01010101"; 
         C3_IN <= "11110000"; 
         C4_IN <= "11001100"; 
         C5_IN <= "01111110"; 
         C6_IN <= "00110011"; 
         C7_IN <= "11111111"; 
         C8_IN <= "10101010"; 
      END IF;
   END PROCESS;

   -- register input to match pipe-lining with IN_MUX_SEL signals
   
   PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         A_IN_REG <= "00000000";    
         B_IN_REG <= "00000000";    
         C_IN_REG <= "00000000";    
         D_IN_REG <= "00000000";    
         E_IN_REG <= "00000000";    
         F_IN_REG <= "00000000";    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         A_IN_REG <= A_IN;    
         B_IN_REG <= B_IN;    
         C_IN_REG <= C_IN;    
         D_IN_REG <= D_IN;    
         E_IN_REG <= E_IN;    
         F_IN_REG <= F_IN;    
      END IF;
   END PROCESS;

   --
   -- Input mux.
   --
   
   PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         MUX_OUT1A <= "00000000";    
     ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
	  case MUX_SEL12 is
	  when "01" => MUX_OUT1A <= B_IN_REG;
	  when "10" => MUX_OUT1A <= A_IN_REG;
	  when others => MUX_OUT1A <= "00000000";
	  end case;
     END IF;
   END PROCESS;

   PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         MUX_OUT1B <= "00000000";    
   ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
      case MUX_SEL34 is
	  when "01" => MUX_OUT1B <= D_IN_REG;
	  when "10" => MUX_OUT1B <= C_IN_REG;
	  when others =>  MUX_OUT1B <= "00000000";
	  end case;
   END IF;
   END PROCESS;

   PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
        MUX_OUT1C <= "00000000";    
     ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
      case MUX_SEL56 is
	  when "01" => MUX_OUT1C <= F_IN_REG;
	  when "10" => MUX_OUT1C <= E_IN_REG;
	  when others =>  MUX_OUT1C <= "00000000";
	  end case;
	END IF;
   END PROCESS;

        -- MUX_OUT1A <= A_IN_REG WHEN (IN_MUX_SEL1) = '1' ELSE 
        --              B_IN_REG WHEN (IN_MUX_SEL2) = '1' ELSE 
        --              "000000000000000000";    
        -- MUX_OUT1B <= C_IN_REG WHEN (IN_MUX_SEL3) = '1' ELSE 
        --              D_IN_REG WHEN (IN_MUX_SEL4) = '1' ELSE 
        --              "000000000000000000";    
        -- MUX_OUT1C <= E_IN_REG WHEN (IN_MUX_SEL5) = '1' ELSE 
        --              F_IN_REG WHEN (IN_MUX_SEL6) = '1' ELSE 
        --              "000000000000000000";  
PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         MUX_OUT2A <= "00000000";    
         MUX_OUT2B <= "00000000";    
         MUX_OUT <=   "00000000";    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN  
         MUX_OUT2A <= MUX_OUT1A OR MUX_OUT1B;    
         MUX_OUT2B <= MUX_OUT1C;    --  to match pipe stage
         MUX_OUT <= MUX_OUT2A OR MUX_OUT2B;    
      END IF;
   END PROCESS;
   
   -- Shift the output of the mux into a 7 deep 18 wide shift register
   --Input Tap SRL18. Each SRL18_7 has 18 shift registers each 7 bits deep.  
   --One SRL18_7 is used with each DSP48 slice.

   U1 : SRL18_7 
      PORT MAP (CLK => SLICE_CLK, RST => RST, D => MUX_OUT, Q => Q_OUT1);   
   
   U2 : SRL18_7 
      PORT MAP (CLK => SLICE_CLK, RST => RST, D => Q_OUT1, Q => Q_OUT2);   
   
   U3 : SRL18_7 
      PORT MAP (CLK => SLICE_CLK, RST => RST, D => Q_OUT2, Q => Q_OUT3);   
   
   U4 : SRL18_7 
      PORT MAP (CLK => SLICE_CLK, RST => RST, D => Q_OUT3, Q => Q_OUT4);   
   
   U5 : SRL18_7 
      PORT MAP (CLK => SLICE_CLK, RST => RST, D => Q_OUT4, Q => Q_OUT5);   
   
   U6 : SRL18_7 
      PORT MAP (CLK => SLICE_CLK, RST => RST, D => Q_OUT5, Q => Q_OUT6);   
   
   U7 : SRL18_7 
      PORT MAP (CLK => SLICE_CLK, RST => RST, D => Q_OUT6, Q => Q_OUT7);   
   
   U8 : SRL18_7 
      PORT MAP (CLK => SLICE_CLK, RST => RST, D => Q_OUT7, Q => Q_OUT8);   
   
   U21 : FIR_SLICE1 
      PORT MAP (CLK => SLICE_CLK, RST => RST,A_IN => Q_OUT1, C_IN => C1_IN,
                POUT => open, PCOUT => SUM_INT1);   
   
   U22 : FIR_SLICE2 
      PORT MAP (CLK => SLICE_CLK, RST => RST, A_IN => Q_OUT2, C_IN => C2_IN,
                PCIN => SUM_INT1, POUT => open, PCOUT => SUM_INT2);   
   
   U23 : FIR_SLICE2 
      PORT MAP (CLK => SLICE_CLK, RST => RST, A_IN => Q_OUT3, C_IN => C3_IN,
                PCIN => SUM_INT2, POUT => open, PCOUT => SUM_INT3);   
   
   U24 : FIR_SLICE2 
      PORT MAP (CLK => SLICE_CLK, RST => RST, A_IN => Q_OUT4, C_IN => C4_IN,
                PCIN => SUM_INT3, POUT => open, PCOUT => SUM_INT4);   
   
   U25 : FIR_SLICE2 
      PORT MAP (CLK => SLICE_CLK, RST => RST, A_IN => Q_OUT5, C_IN => C5_IN,
                PCIN => SUM_INT4, POUT => open, PCOUT => SUM_INT5);   
   
   U26 : FIR_SLICE2 
      PORT MAP (CLK => SLICE_CLK, RST => RST, A_IN => Q_OUT6, C_IN => C6_IN,
                PCIN => SUM_INT5, POUT => open, PCOUT => SUM_INT6);   
   
   U27 : FIR_SLICE2 
      PORT MAP (CLK => SLICE_CLK, RST => RST, A_IN => Q_OUT7, C_IN => C7_IN,
                PCIN => SUM_INT6, POUT => open, PCOUT => SUM_INT7);   
   
   U28 : FIR_SLICE2 
      PORT MAP (CLK => SLICE_CLK, RST => RST, A_IN => Q_OUT8, C_IN => C8_IN,
                PCIN => SUM_INT7, POUT => SUM_INT8, PCOUT => open);   
   

   -- initial latency counter. The delay is equal to 
   
   --[(number of channels + 1) * number of taps] + 3 pipe stages
   
   PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         DLY_CNTR <= "000000";    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         IF (DLY_CNTR < "111100") THEN
            DLY_CNTR <= DLY_CNTR + "000001";    
         ELSE
            DLY_CNTR <= DLY_CNTR;    
         END IF;
      END IF;
   END PROCESS;

   --output enable is on after the initial delay
   
   PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         OUTPUT_EN <= '0';    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         IF (DLY_CNTR = "111100") THEN
	      OUTPUT_EN <= '1' ;
	    ELSE 
		 OUTPUT_EN <= '1' ; 
	    END IF;   
      END IF;
   END PROCESS;

   PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         SUM_INT8_REG <= "0000000000";    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         SUM_INT8_REG <= SUM_INT8(9 DOWNTO 0);    
      END IF;
   END PROCESS;
   
   -- Output mux
   
   --
   
   PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         OUT_MUX_SEL1 <= '0';    
         OUT_MUX_SEL2 <= '0';    
         OUT_MUX_SEL3 <= '0';    
         OUT_MUX_SEL4 <= '0';    
         OUT_MUX_SEL5 <= '0';    
         OUT_MUX_SEL6 <= '1';    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         OUT_MUX_SEL1 <= OUT_MUX_SEL6;    
         OUT_MUX_SEL2 <= OUT_MUX_SEL1;    
         OUT_MUX_SEL3 <= OUT_MUX_SEL2;    
         OUT_MUX_SEL4 <= OUT_MUX_SEL3;    
         OUT_MUX_SEL5 <= OUT_MUX_SEL4;    
         OUT_MUX_SEL6 <= OUT_MUX_SEL5;    
      END IF;
   END PROCESS;
   OUT_SEL <= "00" & OUT_MUX_SEL6 & OUT_MUX_SEL5 & OUT_MUX_SEL4 & 
   OUT_MUX_SEL3 & OUT_MUX_SEL2 & OUT_MUX_SEL1 ;
   

PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         OUT1 <= "0000000000";    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         IF (OUTPUT_EN = '1' AND OUT_MUX_SEL1 = '1') THEN
            OUT1 <= SUM_INT8_REG(9 DOWNTO 0); 
	    ELSE 
            OUT1 <= "0000000000";               
         END IF;
      END IF;      
END PROCESS;
  
PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         OUT2 <= "0000000000";    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         IF (OUTPUT_EN = '1' AND OUT_MUX_SEL2 = '1') THEN
            OUT2 <= SUM_INT8_REG(9 DOWNTO 0); 
	    ELSE 
            OUT2 <= "0000000000";    
            
         END IF;
      END IF;        
END PROCESS;

PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         OUT3 <= "0000000000";    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         IF (OUTPUT_EN = '1' AND OUT_MUX_SEL3 = '1') THEN
            OUT3 <= SUM_INT8_REG(9 DOWNTO 0); 
	    ELSE 
            OUT3 <= "0000000000";               
         END IF;
      END IF;        
END PROCESS;

PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         OUT4 <= "0000000000";    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         IF (OUTPUT_EN = '1' AND OUT_MUX_SEL4 = '1') THEN
            OUT4 <= SUM_INT8_REG(9 DOWNTO 0); 
	    ELSE 
            OUT4 <= "0000000000";              
         END IF;
      END IF;        
END PROCESS;

PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         OUT5 <= "0000000000";    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         IF (OUTPUT_EN = '1' AND OUT_MUX_SEL5 = '1') THEN
            OUT5 <= SUM_INT8_REG(9 DOWNTO 0); 
	    ELSE 
            OUT5 <= "0000000000";                
         END IF;
      END IF;        
END PROCESS;

PROCESS (SLICE_CLK, RST)
   BEGIN
      IF (RST = '1') THEN
         OUT6 <= "0000000000";    
      ELSIF (SLICE_CLK'EVENT AND SLICE_CLK = '1') THEN
         IF (OUTPUT_EN = '1' AND OUT_MUX_SEL6 = '1') THEN
            OUT6 <= SUM_INT8_REG(9 DOWNTO 0); 
	    ELSE 
            OUT6 <= "0000000000";                
         END IF;
      END IF;        
END PROCESS;

--//////////////////////////////////////////////////////////////////////////////

END FIR_MCF6;
